@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import math
 from abc import abstractmethod, ABC
 from enum import IntEnum
 from typing import List, Optional
@@ -9,6 +10,14 @@ from pykotor.common.language import LocalizedString
 from pykotor.common.misc import Game, Color, ResRef
 from pykotor.resource.formats.gff import GFF, GFFStruct, GFFList, GFFContent, read_gff, write_gff
 from pykotor.resource.formats.gff.gff_auto import bytes_gff
+from pykotor.resource.generics.utc import bytes_utc, UTC
+from pykotor.resource.generics.utd import UTD, bytes_utd
+from pykotor.resource.generics.ute import UTE, bytes_ute
+from pykotor.resource.generics.utm import UTM, bytes_utm
+from pykotor.resource.generics.utp import bytes_utp, UTP
+from pykotor.resource.generics.uts import UTS, bytes_uts
+from pykotor.resource.generics.utt import bytes_utt, UTT
+from pykotor.resource.generics.utw import UTW, bytes_utw
 from pykotor.resource.type import ResourceType, SOURCE_TYPES, TARGET_TYPES
 
 
@@ -190,6 +199,12 @@ class GITInstance(ABC):
     ) -> Optional[ResourceType]:
         ...
 
+    @abstractmethod
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        ...
+
     @abstractmethod
     def move(
             self,
@@ -255,6 +270,11 @@ class GITCamera(GITInstance):
     ) -> Optional[ResRef]:
         return None
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return None
+
     def extension(
             self
     ) -> Optional[ResourceType]:
@@ -302,6 +322,11 @@ class GITCreature(GITInstance):
     ) -> Optional[ResRef]:
         return self.resref
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utc(UTC())
+
     def extension(
             self
     ) -> Optional[ResourceType]:
@@ -331,7 +356,7 @@ class GITDoor(GITInstance):
         super().__init__(x, y, z)
         self.resref: ResRef = ResRef.from_blank()
         self.bearing: float = 0.0
-        self.tweak_color: Optional[Color] = Color.WHITE
+        self.tweak_color: Optional[Color] = None
         self.linked_to: str = ""
         self.linked_to_flags: GITModuleLink = GITModuleLink.NoLink
         self.linked_to_module: ResRef = ResRef.from_blank()
@@ -356,6 +381,11 @@ class GITDoor(GITInstance):
     ) -> None:
         self.bearing += yaw
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utd(UTD())
+
     def reference(
             self
     ) -> Optional[ResRef]:
@@ -379,7 +409,9 @@ class GITEncounterSpawnPoint:
             y: float = 0.0,
             z: float = 0.0
     ):
-        super().__init__(x, y, z)
+        self.x = x
+        self.y = y
+        self.z = z
         self.orientation: float = 0.0
 
 
@@ -422,6 +454,11 @@ class GITEncounter(GITInstance):
     ) -> Optional[ResRef]:
         return self.resref
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_ute(UTE())
+
     def extension(
             self
     ) -> Optional[ResourceType]:
@@ -445,7 +482,7 @@ class GITPlaceable(GITInstance):
         super().__init__(x, y, z)
         self.resref: ResRef = ResRef.from_blank()
         self.bearing: float = 0.0
-        self.tweak_color: Optional[Color] = Color.WHITE
+        self.tweak_color: Optional[Color] = None
 
     def move(
             self,
@@ -470,6 +507,11 @@ class GITPlaceable(GITInstance):
     ) -> Optional[ResRef]:
         return self.resref
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utp(UTP())
+
     def extension(
             self
     ) -> Optional[ResourceType]:
@@ -521,6 +563,11 @@ class GITSound(GITInstance):
     ) -> Optional[ResRef]:
         return self.resref
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_uts(UTS())
+
     def classification(
             self
     ) -> str:
@@ -567,6 +614,11 @@ class GITStore(GITInstance):
             self
     ) -> Optional[ResourceType]:
         return ResourceType.UTM
+
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utm(UTM())
     
     def classification(
             self
@@ -621,6 +673,11 @@ class GITTrigger(GITInstance):
     ) -> Optional[ResourceType]:
         return ResourceType.UTT
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utt(UTT())
+
     def classification(
             self
     ) -> str:
@@ -654,6 +711,7 @@ class GITWaypoint(GITInstance):
         self.name: LocalizedString = LocalizedString.from_invalid()
         self.map_note: Optional[LocalizedString] = LocalizedString.from_invalid()
         self.map_note_enabled: bool = False
+        self.has_map_note: bool = False
         self.bearing: float = 0.0
 
     def move(
@@ -684,6 +742,11 @@ class GITWaypoint(GITInstance):
     ) -> Optional[ResourceType]:
         return ResourceType.UTW
 
+    def blank(
+            self
+    ) -> Optional[bytes]:
+        return bytes_utw(UTW())
+
     def classification(
             self
     ) -> str:
@@ -724,7 +787,7 @@ def construct_git(
         creature.position.y = creature_struct.acquire("YPosition", 0.0)
         creature.position.z = creature_struct.acquire("ZPosition", 0.0)
         rot_x, rot_y = creature_struct.acquire("XOrientation", 0.0), creature_struct.acquire("YOrientation", 0.0)
-        creature.bearing = Vector2(rot_x, rot_y).angle()
+        creature.bearing = Vector2(rot_x, rot_y).angle() - math.pi/2
 
     for door_struct in gff.root.get_list("Door List"):
         door = GITDoor()
@@ -800,7 +863,7 @@ def construct_git(
         store.position.z = store_struct.acquire("ZPosition", 0.0)
 
         rot_x, rot_y = store_struct.acquire("XOrientation", 0.0), store_struct.acquire("YOrientation", 0.0)
-        store.bearing = Vector2(rot_x, rot_y).angle()
+        store.bearing = Vector2(rot_x, rot_y).angle() - math.pi/2
 
     for trigger_struct in gff.root.get_list("TriggerList"):
         trigger = GITTrigger()
@@ -833,13 +896,13 @@ def construct_git(
         waypoint.position.y = waypoint_struct.acquire("YPosition", 0.0)
         waypoint.position.z = waypoint_struct.acquire("ZPosition", 0.0)
 
-        has_map_note = waypoint_struct.acquire("HasMapNote", 0)
-        if has_map_note:
+        waypoint.has_map_note = waypoint_struct.acquire("HasMapNote", 0)
+        if waypoint.has_map_note:
             waypoint.map_note = waypoint_struct.acquire("MapNote", LocalizedString.from_invalid())
             waypoint.map_note_enabled = waypoint_struct.acquire("MapNoteEnabled", 0)
 
         rot_x, rot_y = waypoint_struct.acquire("XOrientation", 0.0), waypoint_struct.acquire("YOrientation", 0.0)
-        waypoint.bearing = Vector2(rot_x, rot_y).angle()
+        waypoint.bearing = Vector2(rot_x, rot_y).angle() - math.pi/2
 
     return git
 
@@ -858,8 +921,11 @@ def dismantle_git(
     properties_struct = root.set_struct("AreaProperties", GFFStruct())
     properties_struct.set_int32("AmbientSndDayVol", git.ambient_volume)
     properties_struct.set_int32("AmbientSndDay", git.ambient_sound_id)
+    properties_struct.set_int32("AmbientSndNitVol", git.ambient_volume)
+    properties_struct.set_int32("AmbientSndNight", git.ambient_sound_id)
     properties_struct.set_int32("EnvAudio", git.env_audio)
-    properties_struct.set_uint32("MusicDay", git.music_standard_id)
+    properties_struct.set_int32("MusicDay", git.music_standard_id)
+    properties_struct.set_int32("MusicNight", git.music_standard_id)
     properties_struct.set_int32("MusicBattle", git.music_battle_id)
     properties_struct.set_int32("MusicDelay", git.music_delay)
 
@@ -876,7 +942,7 @@ def dismantle_git(
 
     creature_list = root.set_list("Creature List", GFFList())
     for creature in git.creatures:
-        bearing = Vector2.from_angle(creature.bearing)
+        bearing = Vector2.from_angle(creature.bearing + math.pi/2)
 
         creature_struct = creature_list.add(GITCreature.GFF_STRUCT_ID)
         creature_struct.set_resref("TemplateResRef", creature.resref)
@@ -923,9 +989,9 @@ def dismantle_git(
         for spawn in encounter.spawn_points:
             spawn_struct = spawn_list.add(GITEncounter.GFF_SPAWN_STRUCT_ID)
             spawn_struct.set_single("Orientation", spawn.orientation)
-            spawn_struct.set_single("X", spawn.position.x)
-            spawn_struct.set_single("Y", spawn.position.y)
-            spawn_struct.set_single("Z", spawn.position.z)
+            spawn_struct.set_single("X", spawn.x)
+            spawn_struct.set_single("Y", spawn.y)
+            spawn_struct.set_single("Z", spawn.z)
 
     placeable_list = root.set_list("Placeable List", GFFList())
     for placeable in git.placeables:
@@ -951,7 +1017,7 @@ def dismantle_git(
 
     store_list = root.set_list("StoreList", GFFList())
     for store in git.stores:
-        bearing = Vector2.from_angle(store.bearing)
+        bearing = Vector2.from_angle(store.bearing + math.pi/2)
 
         store_struct = store_list.add(GITStore.GFF_STRUCT_ID)
         store_struct.set_resref("ResRef", store.resref)
@@ -987,7 +1053,7 @@ def dismantle_git(
 
     waypoint_list = root.set_list("WaypointList", GFFList())
     for waypoint in git.waypoints:
-        bearing = Vector2.from_angle(waypoint.bearing)
+        bearing = Vector2.from_angle(waypoint.bearing + math.pi/2)
 
         waypoint_struct = waypoint_list.add(GITWaypoint.GFF_STRUCT_ID)
 
@@ -1000,9 +1066,8 @@ def dismantle_git(
         waypoint_struct.set_single("XOrientation", bearing.x)
         waypoint_struct.set_single("YOrientation", bearing.y)
         waypoint_struct.set_uint8("MapNoteEnabled", waypoint.map_note_enabled)
-        waypoint_struct.set_uint8("HasMapNote", 0 if waypoint.map_note is None else 1)
-        waypoint_struct.set_locstring("MapNote",
-                                      LocalizedString.from_invalid() if waypoint.map_note is None else waypoint.map_note)
+        waypoint_struct.set_uint8("HasMapNote", waypoint.has_map_note)
+        waypoint_struct.set_locstring("MapNote", LocalizedString.from_invalid() if waypoint.map_note is None else waypoint.map_note)
 
     return gff
 
